from Products.PloneTestCase import PloneTestCase
from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod
from Products.CMFCore.utils import getToolByName

#PloneTestCase.installProduct('ExternalMethod')  # Seems to make no difference.
PloneTestCase.setupPloneSite()


class TestCustomNavtree(PloneTestCase.PloneTestCase):
    def afterSetUp(self):
        def makeTree(node, container):
            container.invokeFactory(type_name=node['type_name'], id=node['id'], title=node.get('Title'))
            for child in node.get('children', []):
                makeTree(child, container[node['id']])
        
        self.loginAsPortalOwner()
        makeTree({'id': 'misterfolder', 'type_name': 'Folder', 'title': 'Mister Folder', 'children':
                    [{'id': 'missfolder', 'type_name': 'Folder', 'title': 'Miss Folder', 'children':
                        [{'id': 'kidfolder', 'type_name': 'Folder', 'title': 'Kid Folder', 'children':
                            [{'id': 'kidderfolder', 'type_name': 'Folder', 'title': 'Kidder Folder', 'children':
                                [{'id': 'kiddestfolder', 'type_name': 'Folder', 'title': 'Kiddest Folder'}]},
                             {'id': 'kidspage', 'type_name': 'Document', 'title': "Kid's Page"}]}]},
                     {'id': 'misterspage', 'type_name': 'Document', 'title': "Mister's Page"}]}, self.portal)
        manage_addExternalMethod(self.portal, 'customNavtree', 'customNavtree', 'CustomNav.nav', 'customNavtree')
        self.logout()
    
    def _failUnlessShowingStuffInMisterFolderOnly(self, tree):
        """Make sure it's showing only misterfolder's children."""
        firstLevelChildren = tree['children']
        self.failUnless(len(firstLevelChildren) == 2)
        self._assertContract(firstLevelChildren[0], 'missfolder', False, False, 1)
        self._assertContract(firstLevelChildren[1], 'misterspage', False, False, 1)
    
    def testFromAboveBasePath(self):
        tree = self.portal.customNavtree(self.portal, basePath='/misterfolder')
        self._failUnlessShowingStuffInMisterFolderOnly(tree)

    def testFromAtBasePath(self):
        tree = self.portal.customNavtree(self.portal.misterfolder, basePath='/misterfolder')
        self._failUnlessShowingStuffInMisterFolderOnly(tree)
    
    def _assertContract(self, node, id, currentItem, currentParent, depth):
        """Fail unless node has the given id, currentItem, currentParent, and depth (which are the only keys customNavtree() promises)."""
        self.failUnless(node['item'].id == id)
        self.failUnless(node['currentItem'] == currentItem)
        self.failUnless(node['currentParent'] == currentParent)
        self.failUnless(node['depth'] == depth)
    
    def testFrom1BelowBasePath(self):
        """When you're in missfolder, missfolder's children should be disclosed, and missfolder should be highlighted."""
        tree = self.portal.customNavtree(self.portal.misterfolder.missfolder, basePath='/misterfolder')
        firstLevelChildren = tree['children']
        missFolder = firstLevelChildren[0]
        self._assertContract(missFolder, 'missfolder', True, False, 1)
        self._assertContract(firstLevelChildren[1], 'misterspage', False, False, 1)
        self._assertContract(missFolder['children'][0], 'kidfolder', False, False, 2)

    def testFrom3BelowBasePath(self):
        """When you're in kidderfolder, its and kidfolder's children should be disclosed, and kidderfolder should be highlighted."""
        tree = self.portal.customNavtree(self.portal.misterfolder.missfolder.kidfolder.kidderfolder, basePath='/misterfolder')
        firstLevelChildren = tree['children']
        missFolder = firstLevelChildren[0]
        self._assertContract(missFolder, 'missfolder', False, True, 1)
        self._assertContract(firstLevelChildren[1], 'misterspage', False, False, 1)
        kidFolder = missFolder['children'][0]
        self._assertContract(kidFolder, 'kidfolder', False, True, 2)
        kidderFolder = kidFolder['children'][0]
        self._assertContract(kidderFolder, 'kidderfolder', True, False, 3)
        kiddestFolder = kidderFolder['children'][0]
        self._assertContract(kiddestFolder, 'kiddestfolder', False, False, 4)
        kidsPage = kidFolder['children'][1]
        self._assertContract(kidsPage, 'kidspage', False, False, 3)
    
    def testDefaultTypeHiding(self):
        """Make sure we inherit the type-hiding behavior of the default navtree: that is, obeying navtree_properties.
        
        If this works, we assume the workflow-filtering and other stuff is inherited as well.
        """
        navtreeProperties = self.portal.portal_properties.navtree_properties
        original = navtreeProperties.metaTypesNotToList
        navtreeProperties.metaTypesNotToList = navtreeProperties.metaTypesNotToList + ('Document',)  # Quit showing Documents.
        try:
            tree = self.portal.customNavtree(self.portal.misterfolder, basePath='/misterfolder')
            self.failUnless(len(tree['children']) == 1)  # Should be showing Miss Folder but not Mister's Page
        finally:
            navtreeProperties.metaTypesNotToList = original  # Put it back how I found it.
    
    def testCustomTypeHiding(self):
        """See if the `types` arg works."""
        tree = self.portal.customNavtree(self.portal.misterfolder, basePath='/misterfolder', types=['Document'])
        firstLevelChildren = tree['children']
        self.failUnless(len(firstLevelChildren) == 1)
        self._assertContract(firstLevelChildren[0], 'misterspage', False, False, 1)
    
    def testDefaultBasePath(self):
        """Test the default case: basePath='.'"""
        tree = self.portal.customNavtree(self.portal.misterfolder)
        firstLevelChildren = tree['children']
        self._assertContract(firstLevelChildren[0], 'missfolder', False, False, 1)
        self._assertContract(firstLevelChildren[1], 'misterspage', False, False, 1)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCustomNavtree))
    return suite
